<?php
/**
 * MapBrowser configuration file
 *
 * @project     MapLab
 * @revision    $Id: globprefs.php,v 1.46.2.3 2004/07/15 19:43:07 zak Exp $
 * @purpose     This file contains the global configurations for MapBrowser.
 * @author      William A. Bronsema, C.E.T. (bronsema@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


// define the path to the conguration settings
define( "CONFIG_PATH", "../../config/" );

// define the path to the components folder
define( "COMMON", "../common/" );

// include the logging functions
include_once( COMMON."logger/logfile.php" );
include_once( COMMON."logger/error_manager.php" );
include_once( COMMON."logger/logger.php" );

// include the xml parsing class
include_once( COMMON."appcontext/appcontext.php" );

//wrapper base class for mapsession
include_once( COMMON."wrapper/map_session.php");

// include WMFDatabase class def
include_once( COMMON."phpwms/dbf.php");
include_once( COMMON."phpwms/server_data_manager.php");

include_once( COMMON."session/session.php" );
installSessionDirectoryHandler();
initializeSession();

$gbVoidSession = false;

// check if the IP has been set and validate
if ( isset( $_SESSION["gszRemoteAdd"] ) && 
                                strlen(trim($_SESSION["gszRemoteAdd"])) > 0 )
{
    // check if IP matches current client
    if ( trim( $_SESSION["gszRemoteAdd"] ) != trim( $_SERVER["REMOTE_ADDR"] ) )
    {
        // possible security breach void session
        $gbVoidSession = true;
    }
}
else
{
    // possible security breach void session
    $gbVoidSession = true;
}

// convert the global session reference if necessary
if ( PHP_OS == "WINNT" || PHP_OS == "WIN32" )
{
    if ( version_compare( phpversion(), "4.2.1" ) < 0 )
        $_SESSION = &$HTTP_SESSION_VARS;
}

// define a new AppContext object
$gContext = new AppContext( CONFIG_PATH."mapbrowser.xml" );

//read context values out of current context object
// main application directory
$gszAppPath = str_replace("\\", "/", realpath($gContext->getContextValue("app_path")));
if (substr($gszAppPath, -1) != "/")
    $gszAppPath .= "/";

// wmsparse.exe file
//$gszWMSParseFile = $gContext->getContextValue("wmsparse_file");
if (!isset( $_SESSION["gszWMSParseFile"] ))
{
    if (PHP_OS == "WINNT" || PHP_OS == "WIN32")
    {
        $_SESSION["gszWMSParseFile"] = "../wmsparse/win32/wmsparse.exe";
    }
    else
    {
        $_SESSION["gszWMSParseFile"] = "../wmsparse/wmsparse";
    }
}

// mapscript module
$gszMapscriptModule = $gContext->getContextValue("mapscript_module");
if (!isset($_SESSION["gszPHPMapScriptModName"]))
    $_SESSION["gszPHPMapScriptModName"] = $gszMapscriptModule;

// image path and url
$gszTmpImgPath = str_replace("\\", "/", realpath($gContext->getContextValue("tmp_img_path")));
if (substr($gszTmpImgPath, -1) != "/")
    $gszTmpImgPath .= "/";
if (!isset($_SESSION["gszTmpImgPath"]))
    $_SESSION["gszTmpImgPath"] = $gszTmpImgPath;
$gszTmpWebImgPath = str_replace("\\", "/", $gContext->getContextValue("tmp_web_img_path"));
if (substr($gszTmpWebImgPath, -1) != "/")
    $gszTmpWebImgPath .= "/";
if (!isset($_SESSION["gszTmpWebImgPath"]))
    $_SESSION["gszTmpWebImgPath"] = $gszTmpWebImgPath;    

// path for the server data directory
//$gszServerDataPath = $gContext->getContextValue("server_data_path");
if (!isset($_SESSION["gszServerDataPath"]))
    $_SESSION["gszServerDataPath"] = $gszAppPath."data/";

define( "SRS_DIR", $_SESSION["gszServerDataPath"] );
define( "ABSTRACT_DIR", $_SESSION["gszServerDataPath"] );

// path for the temp data directory
$gszTmpMapPath = str_replace("\\", "/", realpath($gContext->getContextValue("tmp_map_path")));
if (substr($gszTmpMapPath, -1) != "/")
    $gszTmpMapPath .= "/";
if (!isset($_SESSION["gszTmpMapPath"]))
    $_SESSION["gszTmpMapPath"] = $gszTmpMapPath;    

// path for the file browser root directory
$gszFileBrowserRoot = str_replace("\\", "/", realpath($gContext->getContextValue("file_browser_root")));
if (substr($gszFileBrowserRoot, -1) != "/")
    $gszFileBrowserRoot .= "/";

$_SESSION['gszFileBrowserRoot'] = $gszFileBrowserRoot;

// path and filename of the template to use when building a new map
//$gszMapTemplate = $gContext->getContextValue("map_template");
$gszMapTemplate = $gszAppPath."htdocs/mapbrowser/template/template.map";

//path and filename of demo mode map file
$gszDemoMap = $gszAppPath."htdocs/mapbrowser/template/demo_mode.map";

// navigation mode default (either ROSA or HTML)
$gszDefaultMode = $gContext->getContextValue("default_navigation_mode");

/// Group name that mapbrowser should use for newly created file to.
$gszGroupName = $gContext->getContextValue("group_name");

/// Get default repository
$gszSchemasLocation = $gContext->getContextValue("ows_schemas_location");

/// Get new file permission mask
$gszPermissionMask = $gContext->getContextValue("new_file_permission");
if (strlen($gszPermissionMask) != 4 ||
    $gszPermissionMask[0] != "0" || // first digit must be 0
    $gszPermissionMask[1] < 6 ||    // second must be 6 or 7
    $gszPermissionMask[1] > 7 ||    // ...
    $gszPermissionMask[2] < 0 ||    // Last two digits must be
    $gszPermissionMask[2] > 7 ||    // 0 to 7
    $gszPermissionMask[3] < 0 ||    // ...
    $gszPermissionMask[3] > 7)      // ...
  $gszPermissionMask = "0644";


// path for the log file
$gszLogPath = str_replace("\\", "/", realpath($gContext->getContextValue("log_path")));
if (substr($gszLogPath, -1) != "/")
    $gszLogPath .= "/";

// get the log level
$gszLogLevel = $gContext->getContextValue("log_level");

// create new logfile object (customize parameters)
//$gLogFile = new LogFile( $gszLogPath."mapbrowser.log", LOG_TO_FILE, 0 );
$aszLog = explode(".", basename($_SERVER["PHP_SELF"]));
$gLogFile = new LogFile( $gszLogPath.$aszLog[0]."_mapbrowser.log", LOG_TO_FILE, true );

// rosa file
define( "ROSA_FILE", "./rosa/rosa.jar" );

// define the zoom out factor to be used
define( "ZOOMOUT_FACTOR", 2 );

// define the zoom in factor to be used
define( "ZOOMIN_FACTOR", 2 );

// define the smallest rectangle size (in pixels) for zooming in,
// otherwise treat as a point
define( "MIN_RECT", 5 );

// determine the max log level ( set in mapbrowser.xml )
switch( $gszLogLevel )
{
    case "LOG_ALL";
        $gnLogLevel = LOG_ALL;
        break;
    case "LOG_VERBOSE":
        $gnLogLevel = LOG_VERBOSE;
        break;
    case "LOG_QUIET":
        $gnLogLevel = LOG_QUIET;
        break;
    default:
        $gnLogLevel = LOG_OFF;
} // switch

// set the max log level
$gLogFile->setMaxLogLevel( $gnLogLevel );

if (!isset($_SESSION["gErrorManager"] ) )
{

    // create new error manager object
    $_SESSION["gErrorManager"]= new ErrorManager();
}

// create a logger object
$oLog = new Logger( "General" );

// set the global log file and error manager
$oLog->setLogFile( $gLogFile );
$oLog->setErrorManager( $_SESSION["gErrorManager"] );

//mass register session vars
$aszSessionVars = array( "gnIsTemplate",
                         "gszMaxExtents",
                         "gszCustExtents",
                         "gszCurrentState",
                         "gszPrevStateID",
                         "gnExecMode|getContextValue|\$gContext,\"execution_mode\"",
                         "gszNavigationMode|$gszDefaultMode",
                         "gszMap",
                         "gszMapName",
                         "gszSchemasLocation",
                         "gszMapTemplate"
                       );

foreach($aszSessionVars as $mValue)
{
    $amValues = explode( "|", $mValue );
    if ( !isset($_SESSION[$amValues[0]]))
    {
//        $gLogFile->logMessage( LOG_QUIET, "$szSessionVar is not registered");
        if ( count( $amValues ) == 3 )
        {
            $szEval = "\$mDefault = ".$amValues[1]."(".$amValues[2].");";
            eval( $szEval );
        }
        elseif (count( $amValues ) == 2 )
        {
            $mDefault = $amValues[1];
        }
        else
        {
            $mDefault = "";
        }
        $_SESSION[$amValues[0]] = $mDefault;
    }
    else
    {
        $gLogFile->logMessage( LOG_VERBOSE, $amValues[0]." is ".
                               $_SESSION[$amValues[0]] );
    }
}
/// MapEdit execution mode (0 -> DEMO, 1 -> NORMAL, 2 -> EXPERT)
define ( "DEMO_MODE", 0 );
define ( "NORMAL_MODE", 1 );
define ( "EXPERT_MODE", 2 );

if ($_SESSION["gnExecMode"] === false  ||
    $_SESSION["gnExecMode"] < 0 ||
    $_SESSION["gnExecMode"] > 2 )
{
    $_SESSION["gnExecMode"] = NORMAL_MODE;
}

//handle demo mode
if ($_SESSION["gnExecMode"] == DEMO_MODE)
{
    //things to do:
    //create a working directory for the user
    // and copy in data files
    //remap the data path to the user's directory
    //timestamp the directory
    //remove any other directories that are expired
    //set the template map as the current map

    //working tmp directory created under the existing DATA directory (it has to be
    //writeable anyway).
    $szTmpDataPath = $gszAppPath."data/".session_id()."/";

    //copy in data files
    if (!is_dir($szTmpDataPath))
    {
        //create the directory
        $oLog->log( LOG_VERBOSE, "tmp dir doesn't exist, creating it at $szTmpDataPath");
        mkdir( $szTmpDataPath, 0775 );
        //copy in data files
        if ($hDir = opendir($_SESSION["gszServerDataPath"]))
        {
            while ( ($szFile = readdir($hDir)) !== false)
            {
                if ( is_file($_SESSION["gszServerDataPath"].$szFile) )
                {
                    copy( $_SESSION["gszServerDataPath"].$szFile, $szTmpDataPath.$szFile );
                }
            }
            closedir($hDir);
        }
        else
        {
            $oLog->error( ERR_WARNING, "Failed to open directory $gszServerDataPath" );
        }
    }

    //timestamp the directory
    touch( $szTmpDataPath."timestamp.txt" );

    $oLog->log( LOG_ALL, "touching $szTmpDataPath in ".$_SERVER["PHP_SELF"] );
    //remove other directories
    $aszRemoveDirs = array();
    if ($hDir = opendir($_SESSION["gszServerDataPath"]))
    {
        $szFile = readdir($hDir);
        while ( $szFile  !== false)
        {
            if ($szFile != "." && $szFile != ".." &&
                is_dir($_SESSION["gszServerDataPath"].$szFile) )
            {
                $szTimestamp = $_SESSION["gszServerDataPath"].$szFile."/timestamp.txt";
                if (is_file($szTimestamp))
                {
                    //time returns the current time in seconds since the
                    //Unix Epoch.  This calculates a time 1 hour before now
                    $expiry = time() - 10 * 60;
                    if (filemtime($szTimestamp) < $expiry)
                    {
                        array_push($aszRemoveDirs, $_SESSION["gszServerDataPath"].$szFile."/");
                    }
                }
            }
            $szFile = readdir($hDir);
        }
        closedir($hDir);
        foreach($aszRemoveDirs as $szDir)
            remove_tmp_dir( $szDir, $oLog );
    }

    //remap global data path to user's data directory
    $_SESSION["gszServerDataPath"] = $szTmpDataPath;

    //set the template map name
    if ( $_SESSION["gszMapName"] == "")
    {
        $_SESSION["gszMapName"] = $gszDemoMap;
    }

    //update the contents of the session map variable
    if ( $_SESSION["gszMap"] == "")
    {
        $_SESSION["gszMap"] = writeMapFileString( $_SESSION["gszMapName"], $oLog );
    }
}

//This section defines the location of the database files
$oWMSDatabase = new WMSDatabase($_SESSION["gszServerDataPath"]);

/* ============================================================================
 * Update the current map state
 * ========================================================================= */
if (isset( $_SESSION["gszMap"] ) && $_SESSION["gszMap"] != "" &&
    isset( $_SESSION["gszMapName"] ) && $_SESSION["gszMapName"] != ""
    && $_SESSION["gszCurrentState"] == "")
{
    $_SESSION["gszCurrentState"] = time()."-".rand(1000,9999);
    if (strtoupper(substr($_SESSION["gszMapName"], -3)) == "XML")
        $fp = fopen( $gszTmpMapPath.$_SESSION["gszCurrentState"].".xml", "w" );
    else
        $fp = fopen( $gszTmpMapPath.$_SESSION["gszCurrentState"].".map", "w" );

    fwrite($fp, $_SESSION["gszMap"] );
    fclose($fp);
    //addID( $_SESSION["gszCurrentState"] );
}

//absolute url to base directory
$gszURI = "http://".$_SERVER['HTTP_HOST'].
         dirname($_SERVER['PHP_SELF']);

/**
 * utility function to get a value out of a context, turns all \ to /
 */
function getContextValue( $oContext, $szObject )
{
    $szResult = str_replace("\\", "/",
                    $oContext->getContextValue($szObject));
    return $szResult;
}

/**
 * utility function to get a path value out of a context, adds the trailing / by default
 * and turns all \ to /
 */
function getContextPath( $oContext, $szObject )
{
    $szResult = str_replace("\\", "/",
                    $oContext->getContextValue($szObject));
    if (substr($szResult, -1) != "/")
        $szResult .= "/";
    return $szResult;
}

function addID( $szID )
{
   global $gszPrevStateID;
   global $gszTmpMapPath;

    // push the id onto the string
    if ( strlen( trim( $gszPrevStateID ) ) > 0 ) $gszPrevStateID .= ",";
    $gszPrevStateID .= $szID;

    // create array
    $aszIDs = explode( ",", $gszPrevStateID );

    // get number of items
    $nCount = intval( count( $aszIDs ) );

    // loop through array and remove all unused IDs
    for ( $i=0; $i < $nCount - 5; $i++ )
    {
        // shift the element off the array
        $szTmpID = array_shift( $aszIDs );

        // look for and delete
        if ( file_exists( $gszTmpMapPath.$szTmpID.".map" ) )
            unlink( $gszTmpMapPath.$szTmpID.".map" );
    }

    // empty var
    $gszPrevStateID = "";

    // copy the array back to the global
    $nCount = intval( count( $aszIDs ) );
    for ( $i=0; $i<$nCount; $i++ )
    {
        $gszPrevStateID .= ",".$aszIDs[$i];
    }

    // remove the leading ","
    if ( substr( $gszPrevStateID, 0, 1 ) == "," )
        $gszPrevStateID = substr( $gszPrevStateID, 1 );

// end addID function
}

/**
 * This function opens the given mapfile and writes it to a string.
 *
 * @param $szMapFile string - Path and filename of the mapfile to read.
 * @param $oLog object - Logger objects to log events.
 * @return mixed - Contents of file as a string or false if failed.
 **/
function writeMapFileString( $szMapFile, $oLog )
{
    // log function start
    $oLog->logFuncStart( LOG_VERBOSE, "Begin writeMapFileString() function. " );

    // open the file to read
    $fp = @fopen( $szMapFile, "r" );

    // check for failure
    if ( $fp === false )
    {
        // give error and return
        $oLog->error( ERR_WARNING, "Unable to read file [$szMapFile]." );
        return false;
    }

    // read file to string and close
    $szContents = fread($fp, filesize ( $szMapFile ));
    fclose( $fp );

    // log function exit
    $oLog->logFuncEnd( LOG_VERBOSE, "End writeMapFileString() function. " );

    // return contents
    return $szContents;

// end writeMapFileString function
}


/**
 * This function recursively deletes all the files and directories in
 * the directory tree starting at and including the directory passed
 * as a parameter
 *
 * @param $szDirName string - path of the directory to delete
 * @param $oLog object - Logger objects to log events.
 * @return boolean - true if everything was deleted, false if any
 *                   files or directories couldn't be deleted (and they
 *                   will be recorded in the error log).
 **/
function remove_tmp_dir( $szDirName, $oLog )
{
    $bResult = true;
    //sanity check on directory name (must exist)
    if (!is_dir( $szDirName ))
    {
        $oLog->error( ERR_WARNING, "$szDirName is not a directory" );
        $bResult = false;
    }

    //directory exists, try to open it
    if ( ($hDir = opendir( $szDirName )) !== false && $bResult )
    {
        //opened directory, now enumerate deleting files and
        //recursing into sub-directories.
        while(($szFile = readdir($hDir)) !== false && $bResult )
        {
            //skip directory entries for current and parent
            if ( $szFile != "." && $szFile != ".." )
            {
                //add file name to directory
                $szFile = $szDirName."/".$szFile;
                //if its a directory, recurse into it
                if ( is_dir($szFile) )
                {
                    $bResult = remove_tmp_dir( $szFile, $oLog );
                }
                else //otherwise unlink the file
                {
                    //check for errors (probably permissions?)
                    if ( unlink($szFile) == 0 )
                    {
                        $oLog->error( ERR_WARNING, "couldn't delete $szFile" );
                        $bResult = false;
                    }
                    else //success
                    {
                        $oLog->log( LOG_ALL, "removed file $szFile" );
                    }

                }
            }
        }
        //done with the directory,
        closedir($hDir);
        //try to delete the directory
        if( rmdir($szDirName) == 0 )
        {
            $oLog->error( ERR_WARNING, "couldn't delete $szDirName" );
            $bResult = false;
        }
        else //error deleting the directory
        {
            $oLog->log( LOG_ALL, "removed directory $szDirName" );
        }
    }
    else //error opening directory
    {
        $oLog->error( ERR_WARNING, "couldn't open $szDirName");
        $bResult = false;
    }

    // everything worked for this directory
    return $bResult;
}
?>
